/*************************************************************************/ /*!
@File           hal_interface.h
@Title
@Copyright      Copyright (c) Innosilicon Technology Ltd. All Rights Reserved
@Description
@License        Dual MIT/GPLv2

The contents of this file are subject to the MIT license as set out below.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

Alternatively, the contents of this file may be used under the terms of
the GNU General Public License Version 2 ("GPL") in which case the provisions
of GPL are applicable instead of those above.

If you wish to allow use of your version of this file only under the terms of
GPL, and not to allow others to use your version of this file under the terms
of the MIT license, indicate your decision by deleting the provisions above
and replace them with the notice and other provisions required by GPL as set
out in the file called "GPL-COPYING" included in this distribution. If you do
not delete the provisions above, a recipient may use your version of this file
under the terms of either the MIT license or GPL.

This License is also included in this distribution in the file called
"MIT-COPYING".

EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/ /**************************************************************************/

#ifndef __HAL_INTERFACE_H__
#define __HAL_INTERFACE_H__

#include "inno_plat_dev.h"
#include "inno_mm.h"
#include "inno_math.h"
#include "inno_fs.h"
#include "inno_lock.h"

#define DRV_NAME "inno-drv"

#if (defined __G1_PAL__ || defined __G1_SOC__ || defined __G1_NE__)
#define __G1__
#elif (defined __G0_PAL__ || defined __G0_SOC__ || defined __G0_NE__)
#define __G0__
#endif

/*******************DEVICE_NAME*******************/
#define INNO_PMBUS_DEVICE_NAME		"inno-pmbus"
#define INNO_HDMI_DEVICE_NAME		"inno-pdp-hdmi"
#define INNO_DP_DEVICE_NAME			"inno-pdp-dp"
#define INNO_VGA_DEVICE_NAME		"inno-pdp-vga"
#define INNO_DPU_DEVICE_NAME		"inno-pdp-dpu"
#define INNO_VKMS_DEVICE_NAME		"inno-pdp-vkms"
#define INNO_LVDS_DEVICE_NAME		"inno-pdp-lvds"
#define INNO_DRM_DEVICE_NAME		"inno-drm"
#define INNO_AUDIO_DEVICE_NAME		"inno-audio"
#define INNO_CODEC_DEVICE_NAME   	"inno-codec"
#define INNO_DMA_DEVICE_NAME		"inno-dma"
#define INNO_DMAR_DEVICE_NAME		"inno-dmar"
#define INNO_PDMA_DEVICE_NAME		"inno-pdma"
#define INNO_GPU_DEVICE_NAME		"inno-gpu"
#define INNO_AI_SR_DEVICE_NAME		"inno-aisr"
#define INNO_APU_DEVICE_NAME		"inno-apu"
#define INNO_SYSDBG_DEVICE_NAME		"sysdbg"
#define INNO_POWER_DEVICE_NAME		"inno-power"
#define INNO_SMMU_DEVICE_NAME       "inno-smmu"

#define OVERHEAT_NOTIFY (13)
enum {
	INTR_TYPE_GET_ENV = 1,
	INTR_TYPE_SET_ENV,
	INTR_TYPE_OVERHEAT_NOTIFY,
	INTR_TYPE_MAX,
};

/*组件角色的枚举*/
enum vram_alloc_role {
	HAL_VRAM_ROLE_GPU=0,
	HAL_VRAM_ROLE_DPU,
	HAL_VRAM_ROLE_VPU,
	HAL_VRAM_ROLE_APU,
	HAL_VRAM_ROLE_DMA,
	HAL_VRAM_ROLE_AUDIO,
        HAL_VRAM_ROLE_ION,
	HAL_VRAM_ROLE_MAX,
};

/*标识一个组件的角色及对应id的结构体*/
struct role_target {
	enum vram_alloc_role vram_role;
	uint8_t id; /* 主id，比如vpu0*/
	uint8_t sub_id; /* 一般是通道数，比如vpu0下的channel 0~3*/
};

struct vram_stats{
	uint64_t total_size;
	uint64_t free_size;
};

struct disp_interface_info {
	int interface_type;
	int max_hdisp;
	int max_vdisp;
	int max_refresh;
};

typedef enum inno_chip_flags_e {
	INNO_ASIC_MASK = 0x0000ffffUL,
	INNO_FLAGS_MASK  = 0xffff0000UL,
}inno_chip_flags;

typedef enum chip_type_t {
	CHIP_INNO=0,
	CHIP_G1_SOC,
	CHIP_G1_PAL,
	CHIP_G1_NE,
	CHIP_G0_SOC,
	CHIP_G0_PAL,
	CHIP_G0_NE,
	CHIP_G1P_SOC,
	CHIP_G1P_PAL,
	CHIP_G1P_NE,
	CHIP_G0M_SOC,
	CHIP_G0M_PAL,
	CHIP_G0M_NE,
	CHIP_G3_SOC,
	CHIP_G3_PAL,
	CHIP_G3_NE,
	CHIP_TYPE_MAX,
}chip_type_e;

typedef enum dev_module {
	DEV_HDMI=0,
	DEV_DP,
	DEV_VGA,
	DEV_DPU,
	DEV_DB9000,
	DEV_PDP0,
	DEV_VKMS,
	DEV_LVDS,
	DEV_VPU,
	DEV_AUDIO,
	DEV_DMA,
	DEV_PMBUS,
	DEV_GPU,
	DEV_APU,
	DEV_DRM,
	DEV_MODULE_MAX,
}dev_module_e;

typedef enum pll_module {
	PLL_CPU=0,
	PLL_VPU,
	PLL_GPU,
	PLL_DATABUS,
	PLL_APU,
	PLL_DPU,
	PLL_DDR0_1,
	PLL_DDR2_3,
	PLL_CBUS,
	PLL_VGA,
	PLL_LVDS,
	PLL_MODULE_MAX,
}pll_module_e;

enum reg_entity {
	REG_ENTITY0000 = 0,
	REG_ENTITY0001 = 1,
	REG_ENTITY0002 = 2,
	REG_ENTITY0003 = 3,
	REG_ENTITY0004 = 4,
	REG_ENTITY0005 = 5,
	REG_ENTITY0006 = 6,
	REG_ENTITY0007 = 7,
	REG_ENTITY0008 = 8,
	REG_ENTITY0009 = 9,
	REG_ENTITY0010 = 10,
	REG_ENTITY0011 = 11,
	REG_ENTITY0012 = 12,
	REG_ENTITY0013 = 13,
	REG_ENTITY0014 = 14,
	REG_ENTITY0015 = 15,
	REG_ENTITY0016 = 16,
	REG_ENTITY0017 = 17,
	REG_ENTITY0018 = 18,
	REG_ENTITY0019 = 19,
	REG_ENTITY0020 = 20,
	REG_ENTITY0021 = 21,
	REG_ENTITY0022 = 22,
	REG_ENTITY0023 = 23,
	REG_ENTITY0024 = 24,
	REG_ENTITY0025 = 25,
	REG_ENTITY0026 = 26,
	REG_ENTITY0027 = 27,
	REG_ENTITY0028 = 28,
	REG_ENTITY0029 = 29,
	REG_ENTITY0030 = 30,
	REG_ENTITY0031 = 31,
	REG_ENTITY0032 = 32,
	REG_ENTITY0033 = 33,
	REG_ENTITY0034 = 34,
	REG_ENTITY0035 = 35,
	REG_ENTITY0036 = 36,
	REG_ENTITY0037 = 37,
	REG_ENTITY0038 = 38,
	REG_ENTITY0039 = 39,
	REG_ENTITY0040 = 40,
	REG_ENTITY0041 = 41,
	REG_ENTITY0042 = 42,
	REG_ENTITY0043 = 43,
	REG_ENTITY0044 = 44,
	REG_ENTITY0045 = 45,
	REG_ENTITY0046 = 46,
	REG_ENTITY0047 = 47,
	REG_ENTITY0048 = 48,
	REG_ENTITY0049 = 49,
	REG_ENTITY0050 = 50,
	REG_ENTITY0051 = 51,
	REG_ENTITY0052 = 52,
	REG_ENTITY0053 = 53,
	REG_ENTITY0054 = 54,
	REG_ENTITY0055 = 55,
	REG_ENTITY0056 = 56,
	REG_ENTITY0057 = 57,
	REG_ENTITY0058 = 58,
	REG_ENTITY0059 = 59,
	REG_ENTITY0060 = 60,
	REG_ENTITY0061 = 61,
	REG_ENTITY0062 = 62,
	REG_ENTITY0063 = 63,
	REG_ENTITY0064 = 64,
	REG_ENTITY0065 = 65,
	REG_ENTITY0066 = 66,
	REG_ENTITY0067 = 67,
	REG_ENTITY0068 = 68,
	REG_ENTITY0069 = 69,
	REG_ENTITY0070 = 70,
	REG_ENTITY0071 = 71,
	REG_ENTITY0072 = 72,
	REG_ENTITY0073 = 73,
	REG_ENTITY0074 = 74,
	REG_ENTITY0075 = 75,
	REG_ENTITY0076 = 76,
	REG_ENTITY0077 = 77,
	REG_ENTITY0078 = 78,
	REG_ENTITY0079 = 79,
	REG_ENTITY0080 = 80,
	REG_ENTITY0081 = 81,
	REG_ENTITY0082 = 82,
	REG_ENTITY0083 = 83,
	REG_ENTITY0084 = 84,
	REG_ENTITY0085 = 85,
	REG_ENTITY0086 = 86,
	REG_ENTITY0087 = 87,
	REG_ENTITY0088 = 88,
	REG_ENTITY0089 = 89,
	REG_ENTITY0090 = 90,
	REG_ENTITY0091 = 91,
	REG_ENTITY0092 = 92,
	REG_ENTITY0093 = 93,
	REG_ENTITY0094 = 94,
	REG_ENTITY0095 = 95,
	REG_ENTITY0096 = 96,
	REG_ENTITY0097 = 97,
	REG_ENTITY0098 = 98,
	REG_ENTITY0099 = 99,
	REG_ENTITY0100 = 100,
	REG_ENTITY0101 = 101,
	REG_ENTITY0102 = 102,
	REG_ENTITY0103 = 103,
	REG_ENTITY0104 = 104,
	REG_ENTITY0105 = 105,
	REG_ENTITY0106 = 106,
	REG_ENTITY0107 = 107,
	REG_ENTITY0108 = 108,
	REG_ENTITY0109 = 109,
	REG_ENTITY0110 = 110,
	REG_ENTITY0111 = 111,
	REG_ENTITY0112 = 112,
	REG_ENTITY0113 = 113,
	REG_ENTITY0114 = 114,
	REG_ENTITY0115 = 115,
	REG_ENTITY0116 = 116,
	REG_ENTITY0117 = 117,
	REG_ENTITY0118 = 118,
	REG_ENTITY0119 = 119,
	REG_ENTITY0120 = 120,
	REG_ENTITY0121 = 121,
	REG_ENTITY0122 = 122,
	REG_ENTITY0123 = 123,
	REG_ENTITY0124 = 124,
	REG_ENTITY0125 = 125,
	REG_ENTITY0126 = 126,
	REG_ENTITY0127 = 127,
	REG_ENTITY0128 = 128,
	REG_ENTITY0129 = 129,
	REG_ENTITY0130 = 130,
	REG_ENTITY0131 = 131,
	REG_ENTITY0132 = 132,
	REG_ENTITY0133 = 133,
	REG_ENTITY0134 = 134,
	REG_ENTITY0135 = 135,
	REG_ENTITY0136 = 136,
	REG_ENTITY0137 = 137,
	REG_ENTITY0138 = 138,
	REG_ENTITY0139 = 139,
	REG_ENTITY0140 = 140,
	REG_ENTITY0141 = 141,
	REG_ENTITY0142 = 142,
	REG_ENTITY0143 = 143,
	REG_ENTITY0144 = 144,
	REG_ENTITY0145 = 145,
	REG_ENTITY0146 = 146,
	REG_ENTITY0147 = 147,
	REG_ENTITY0148 = 148,
	REG_ENTITY0149 = 149,
	REG_ENTITY0150 = 150,
	REG_ENTITY0151 = 151,
	REG_ENTITY0152 = 152,
	REG_ENTITY0153 = 153,
	REG_ENTITY0154 = 154,
	REG_ENTITY0155 = 155,
	REG_ENTITY0156 = 156,
	REG_ENTITY0157 = 157,
	REG_ENTITY0158 = 158,
	REG_ENTITY0159 = 159,
	REG_ENTITY0160 = 160,
	REG_ENTITY0161 = 161,
	REG_ENTITY0162 = 162,
	REG_ENTITY0163 = 163,
	REG_ENTITY0164 = 164,
	REG_ENTITY0165 = 165,
	REG_ENTITY0166 = 166,
	REG_ENTITY0167 = 167,
	REG_ENTITY0168 = 168,
	REG_ENTITY0169 = 169,
	REG_ENTITY0170 = 170,
	REG_ENTITY0171 = 171,
	REG_ENTITY0172 = 172,
	REG_ENTITY0173 = 173,
	REG_ENTITY0174 = 174,
	REG_ENTITY0175 = 175,
	REG_ENTITY0176 = 176,
	REG_ENTITY0177 = 177,
	REG_ENTITY0178 = 178,
	REG_ENTITY0179 = 179,
	REG_ENTITY0180 = 180,
	REG_ENTITY0181 = 181,
	REG_ENTITY0182 = 182,
	REG_ENTITY0183 = 183,
	REG_ENTITY0184 = 184,
	REG_ENTITY0185 = 185,
	REG_ENTITY0186 = 186,
	REG_ENTITY0187 = 187,
	REG_ENTITY0188 = 188,
	REG_ENTITY0189 = 189,
	REG_ENTITY0190 = 190,
	REG_ENTITY0191 = 191,
	REG_ENTITY0192 = 192,
	REG_ENTITY0193 = 193,
	REG_ENTITY0194 = 194,
	REG_ENTITY0195 = 195,
	REG_ENTITY0196 = 196,
	REG_ENTITY0197 = 197,
	REG_ENTITY0198 = 198,
	REG_ENTITY0199 = 199,
	REG_ENTITY0200 = 200,
	REG_ENTITY0201 = 201,
	REG_ENTITY0202 = 202,
	REG_ENTITY0203 = 203,
	REG_ENTITY0204 = 204,
	REG_ENTITY0205 = 205,
	REG_ENTITY0206 = 206,
	REG_ENTITY0207 = 207,
	REG_ENTITY0208 = 208,
	REG_ENTITY0209 = 209,
	REG_ENTITY0210 = 210,
	REG_ENTITY0211 = 211,
	REG_ENTITY0212 = 212,
	REG_ENTITY0213 = 213,
	REG_ENTITY0214 = 214,
	REG_ENTITY0215 = 215,
	REG_ENTITY0216 = 216,
	REG_ENTITY0217 = 217,
	REG_ENTITY0218 = 218,
	REG_ENTITY0219 = 219,
	REG_ENTITY0220 = 220,
	REG_ENTITY0221 = 221,
	REG_ENTITY0222 = 222,
	REG_ENTITY0223 = 223,
	REG_ENTITY0224 = 224,
	REG_ENTITY0225 = 225,
	REG_ENTITY0226 = 226,
	REG_ENTITY0227 = 227,
	REG_ENTITY0228 = 228,
	REG_ENTITY0229 = 229,
	REG_ENTITY0230 = 230,
	REG_ENTITY0231 = 231,
	REG_ENTITY0232 = 232,
	REG_ENTITY0233 = 233,
	REG_ENTITY0234 = 234,
	REG_ENTITY0235 = 235,
	REG_ENTITY0236 = 236,
	REG_ENTITY0237 = 237,
	REG_ENTITY0238 = 238,
	REG_ENTITY0239 = 239,
	REG_ENTITY0240 = 240,
	REG_ENTITY0241 = 241,
	REG_ENTITY0242 = 242,
	REG_ENTITY0243 = 243,
	REG_ENTITY0244 = 244,
	REG_ENTITY0245 = 245,
	REG_ENTITY0246 = 246,
	REG_ENTITY0247 = 247,
	REG_ENTITY0248 = 248,
	REG_ENTITY0249 = 249,
	REG_ENTITY0250 = 250,
	REG_ENTITY0251 = 251,
	REG_ENTITY0252 = 252,
	REG_ENTITY0253 = 253,
	REG_ENTITY0254 = 254,
	REG_ENTITY0255 = 255,
	REG_ENTITY0256 = 256,
	REG_ENTITY0257 = 257,
	REG_ENTITY0258 = 258,
	REG_ENTITY0259 = 259,
	REG_ENTITY0260 = 260,
	REG_ENTITY0261 = 261,
	REG_ENTITY0262 = 262,
	REG_ENTITY0263 = 263,
	REG_ENTITY0264 = 264,
	REG_ENTITY0265 = 265,
	REG_ENTITY0266 = 266,
	REG_ENTITY0267 = 267,
	REG_ENTITY0268 = 268,
	REG_ENTITY0269 = 269,
	REG_ENTITY0270 = 270,
	REG_ENTITY0271 = 271,
	REG_ENTITY0272 = 272,
	REG_ENTITY0273 = 273,
	REG_ENTITY0274 = 274,
	REG_ENTITY0275 = 275,
	REG_ENTITY0276 = 276,
	REG_ENTITY0277 = 277,
	REG_ENTITY0278 = 278,
	REG_ENTITY0279 = 279,
	REG_ENTITY0280 = 280,
	REG_ENTITY0281 = 281,
	REG_ENTITY0282 = 282,
	REG_ENTITY0283 = 283,
	REG_ENTITY0284 = 284,
	REG_ENTITY0285 = 285,
	REG_ENTITY0286 = 286,
	REG_ENTITY0287 = 287,
	REG_ENTITY0288 = 288,
	REG_ENTITY0289 = 289,
	REG_ENTITY0290 = 290,
	REG_ENTITY0291 = 291,
	REG_ENTITY0292 = 292,
	REG_ENTITY0293 = 293,
	REG_ENTITY0294 = 294,
	REG_ENTITY0295 = 295,
	REG_ENTITY0296 = 296,
	REG_ENTITY0297 = 297,
	REG_ENTITY0298 = 298,
	REG_ENTITY0299 = 299,
	REG_ENTITY0300 = 300,
	REG_ENTITY0301 = 301,
	REG_ENTITY0302 = 302,
	REG_ENTITY0303 = 303,
	REG_ENTITY0304 = 304,
	REG_ENTITY0305 = 305,
	REG_ENTITY0306 = 306,
	REG_ENTITY0307 = 307,
	REG_ENTITY0308 = 308,
	REG_ENTITY0309 = 309,
	REG_ENTITY0310 = 310,
	REG_ENTITY0311 = 311,
	REG_ENTITY0312 = 312,
	REG_ENTITY0313 = 313,
	REG_ENTITY0314 = 314,
	REG_ENTITY0315 = 315,
	REG_ENTITY0316 = 316,
	REG_ENTITY0317 = 317,
	REG_ENTITY0318 = 318,
	REG_ENTITY0319 = 319,
	REG_ENTITY0320 = 320,
	REG_ENTITY0321 = 321,
	REG_ENTITY0322 = 322,
	REG_ENTITY0323 = 323,
	REG_ENTITY0324 = 324,
	REG_ENTITY0325 = 325,
	REG_ENTITY0326 = 326,
	REG_ENTITY0327 = 327,
	REG_ENTITY0328 = 328,
	REG_ENTITY0329 = 329,
	REG_ENTITY0330 = 330,
	REG_ENTITY0331 = 331,
	REG_ENTITY0332 = 332,
	REG_ENTITY0333 = 333,
	REG_ENTITY0334 = 334,
	REG_ENTITY0335 = 335,
	REG_ENTITY0336 = 336,
	REG_ENTITY0337 = 337,
	REG_ENTITY0338 = 338,
	REG_ENTITY0339 = 339,
	REG_ENTITY0340 = 340,
	REG_ENTITY0341 = 341,
	REG_ENTITY0342 = 342,
	REG_ENTITY0343 = 343,
	REG_ENTITY0344 = 344,
	REG_ENTITY0345 = 345,
	REG_ENTITY0346 = 346,
	REG_ENTITY0347 = 347,
	REG_ENTITY0348 = 348,
	REG_ENTITY0349 = 349,
	REG_ENTITY0350 = 350,
	REG_ENTITY0351 = 351,
	REG_ENTITY0352 = 352,
	REG_ENTITY0353 = 353,
	REG_ENTITY0354 = 354,
	REG_ENTITY0355 = 355,
	REG_ENTITY0356 = 356,
	REG_ENTITY0357 = 357,
	REG_ENTITY0358 = 358,
	REG_ENTITY0359 = 359,
	REG_ENTITY0360 = 360,
	REG_ENTITY0361 = 361,
	REG_ENTITY0362 = 362,
	REG_ENTITY0363 = 363,
	REG_ENTITY0364 = 364,
	REG_ENTITY0365 = 365,
	REG_ENTITY0366 = 366,
	REG_ENTITY0367 = 367,
	REG_ENTITY0368 = 368,
	REG_ENTITY0369 = 369,
	REG_ENTITY0370 = 370,
	REG_ENTITY0371 = 371,
	REG_ENTITY0372 = 372,
	REG_ENTITY0373 = 373,
	REG_ENTITY0374 = 374,
	REG_ENTITY0375 = 375,
	REG_ENTITY0376 = 376,
	REG_ENTITY0377 = 377,
	REG_ENTITY0378 = 378,
	REG_ENTITY0379 = 379,
	REG_ENTITY0380 = 380,
	REG_ENTITY0381 = 381,
	REG_ENTITY0382 = 382,
	REG_ENTITY0383 = 383,
	REG_ENTITY0384 = 384,
	REG_ENTITY0385 = 385,
	REG_ENTITY0386 = 386,
	REG_ENTITY0387 = 387,
	REG_ENTITY0388 = 388,
	REG_ENTITY0389 = 389,
	REG_ENTITY0390 = 390,
	REG_ENTITY0391 = 391,
	REG_ENTITY0392 = 392,
	REG_ENTITY0393 = 393,
	REG_ENTITY0394 = 394,
	REG_ENTITY0395 = 395,
	REG_ENTITY0396 = 396,
	REG_ENTITY0397 = 397,
	REG_ENTITY0398 = 398,
	REG_ENTITY0399 = 399,
	REG_ENTITY0400 = 400,
	REG_ENTITY0401 = 401,
	REG_ENTITY0402 = 402,
	REG_ENTITY0403 = 403,
	REG_ENTITY0404 = 404,
	REG_ENTITY0405 = 405,
	REG_ENTITY0406 = 406,
	REG_ENTITY0407 = 407,
	REG_ENTITY0408 = 408,
	REG_ENTITY0409 = 409,
	REG_ENTITY0410 = 410,
	REG_ENTITY0411 = 411,
	REG_ENTITY0412 = 412,
	REG_ENTITY0413 = 413,
	REG_ENTITY0414 = 414,
	REG_ENTITY0415 = 415,
	REG_ENTITY0416 = 416,
	REG_ENTITY0417 = 417,
	REG_ENTITY0418 = 418,
	REG_ENTITY0419 = 419,
	REG_ENTITY0420 = 420,
	REG_ENTITY0421 = 421,
	REG_ENTITY0422 = 422,
	REG_ENTITY0423 = 423,
	REG_ENTITY0424 = 424,
	REG_ENTITY0425 = 425,
	REG_ENTITY0426 = 426,
	REG_ENTITY0427 = 427,
	REG_ENTITY0428 = 428,
	REG_ENTITY0429 = 429,
	REG_ENTITY0430 = 430,
	REG_ENTITY0431 = 431,
	REG_ENTITY0432 = 432,
	REG_ENTITY0433 = 433,
	REG_ENTITY0434 = 434,
	REG_ENTITY0435 = 435,
	REG_ENTITY0436 = 436,
	REG_ENTITY0437 = 437,
	REG_ENTITY0438 = 438,
	REG_ENTITY0439 = 439,
	REG_ENTITY0440 = 440,
	REG_ENTITY0441 = 441,
	REG_ENTITY0442 = 442,
	REG_ENTITY0443 = 443,
	REG_ENTITY0444 = 444,
	REG_ENTITY0445 = 445,
	REG_ENTITY0446 = 446,
	REG_ENTITY0447 = 447,
	REG_ENTITY0448 = 448,
	REG_ENTITY0449 = 449,
	REG_ENTITY0450 = 450,
	REG_ENTITY0451 = 451,
	REG_ENTITY0452 = 452,
	REG_ENTITY0453 = 453,
	REG_ENTITY0454 = 454,
	REG_ENTITY0455 = 455,
	REG_ENTITY0456 = 456,
	REG_ENTITY0457 = 457,
	REG_ENTITY0458 = 458,
	REG_ENTITY0459 = 459,
	REG_ENTITY0460 = 460,
	REG_ENTITY0461 = 461,
	REG_ENTITY0462 = 462,
	REG_ENTITY0463 = 463,
	REG_ENTITY0464 = 464,
	REG_ENTITY0465 = 465,
	REG_ENTITY0466 = 466,
	REG_ENTITY0467 = 467,
	REG_ENTITY0468 = 468,
	REG_ENTITY0469 = 469,
	REG_ENTITY0470 = 470,
	REG_ENTITY0471 = 471,
	REG_ENTITY0472 = 472,
	REG_ENTITY0473 = 473,
	REG_ENTITY0474 = 474,
	REG_ENTITY0475 = 475,
	REG_ENTITY0476 = 476,
	REG_ENTITY0477 = 477,
	REG_ENTITY0478 = 478,
	REG_ENTITY0479 = 479,
	REG_ENTITY0480 = 480,
	REG_ENTITY0481 = 481,
	REG_ENTITY0482 = 482,
	REG_ENTITY0483 = 483,
	REG_ENTITY0484 = 484,
	REG_ENTITY0485 = 485,
	REG_ENTITY0486 = 486,
	REG_ENTITY0487 = 487,
	REG_ENTITY0488 = 488,
	REG_ENTITY0489 = 489,
	REG_ENTITY0490 = 490,
	REG_ENTITY0491 = 491,
	REG_ENTITY0492 = 492,
	REG_ENTITY0493 = 493,
	REG_ENTITY0494 = 494,
	REG_ENTITY0495 = 495,
	REG_ENTITY0496 = 496,
	REG_ENTITY0497 = 497,
	REG_ENTITY0498 = 498,
	REG_ENTITY0499 = 499,
	REG_ENTITY0500 = 500,
	REG_ENTITY0501 = 501,
	REG_ENTITY0502 = 502,
	REG_ENTITY0503 = 503,
	REG_ENTITY0504 = 504,
	REG_ENTITY0505 = 505,
	REG_ENTITY0506 = 506,
	REG_ENTITY0507 = 507,
	REG_ENTITY0508 = 508,
	REG_ENTITY0509 = 509,
	REG_ENTITY0510 = 510,
	REG_ENTITY0511 = 511,
	REG_ENTITY0512 = 512,
	REG_ENTITY0513 = 513,
	REG_ENTITY0514 = 514,
	REG_ENTITY0515 = 515,
	REG_ENTITY0516 = 516,
	REG_ENTITY0517 = 517,
	REG_ENTITY0518 = 518,
	REG_ENTITY0519 = 519,
	REG_ENTITY0520 = 520,
	REG_ENTITY0521 = 521,
	REG_ENTITY0522 = 522,
	REG_ENTITY0523 = 523,
	REG_ENTITY0524 = 524,
	REG_ENTITY0525 = 525,
	REG_ENTITY0526 = 526,
	REG_ENTITY0527 = 527,
	REG_ENTITY0528 = 528,
	REG_ENTITY0529 = 529,
	REG_ENTITY0530 = 530,
	REG_ENTITY0531 = 531,
	REG_ENTITY0532 = 532,
	REG_ENTITY0533 = 533,
	REG_ENTITY0534 = 534,
	REG_ENTITY0535 = 535,
	REG_ENTITY0536 = 536,
	REG_ENTITY0537 = 537,
	REG_ENTITY0538 = 538,
	REG_ENTITY0539 = 539,
	REG_ENTITY0540 = 540,
	REG_ENTITY0541 = 541,
	REG_ENTITY0542 = 542,
	REG_ENTITY0543 = 543,
	REG_ENTITY0544 = 544,
	REG_ENTITY0545 = 545,
	REG_ENTITY0546 = 546,
	REG_ENTITY0547 = 547,
	REG_ENTITY0548 = 548,
	REG_ENTITY0549 = 549,
	REG_ENTITY0550 = 550,
	REG_ENTITY0551 = 551,
	REG_ENTITY0552 = 552,
	REG_ENTITY0553 = 553,
	REG_ENTITY0554 = 554,
	REG_ENTITY0555 = 555,
	REG_ENTITY0556 = 556,
	REG_ENTITY0557 = 557,
	REG_ENTITY0558 = 558,
	REG_ENTITY0559 = 559,
	REG_ENTITY0560 = 560,
	REG_ENTITY0561 = 561,
	REG_ENTITY0562 = 562,
	REG_ENTITY0563 = 563,
	REG_ENTITY0564 = 564,
	REG_ENTITY0565 = 565,
	REG_ENTITY0566 = 566,
	REG_ENTITY0567 = 567,
	REG_ENTITY0568 = 568,
	REG_ENTITY0569 = 569,
	REG_ENTITY0570 = 570,
	REG_ENTITY0571 = 571,
	REG_ENTITY0572 = 572,
	REG_ENTITY0573 = 573,
	REG_ENTITY0574 = 574,
	REG_ENTITY0575 = 575,
	REG_ENTITY0576 = 576,
	REG_ENTITY0577 = 577,
	REG_ENTITY0578 = 578,
	REG_ENTITY0579 = 579,
	REG_ENTITY0580 = 580,
	REG_ENTITY0581 = 581,
	REG_ENTITY0582 = 582,
	REG_ENTITY0583 = 583,
	REG_ENTITY0584 = 584,
	REG_ENTITY0585 = 585,
	REG_ENTITY0586 = 586,
	REG_ENTITY0587 = 587,
	REG_ENTITY0588 = 588,
	REG_ENTITY0589 = 589,
	REG_ENTITY0590 = 590,
	REG_ENTITY0591 = 591,
	REG_ENTITY0592 = 592,
	REG_ENTITY0593 = 593,
	REG_ENTITY0594 = 594,
	REG_ENTITY0595 = 595,
	REG_ENTITY0596 = 596,
	REG_ENTITY0597 = 597,
	REG_ENTITY0598 = 598,
	REG_ENTITY0599 = 599,
	REG_ENTITY0600 = 600,
	REG_ENTITY0601 = 601,
	REG_ENTITY0602 = 602,
	REG_ENTITY0603 = 603,
	REG_ENTITY0604 = 604,
	REG_ENTITY0605 = 605,
	REG_ENTITY0606 = 606,
	REG_ENTITY0607 = 607,
	REG_ENTITY0608 = 608,
	REG_ENTITY0609 = 609,
	REG_ENTITY0610 = 610,
	REG_ENTITY0611 = 611,
	REG_ENTITY0612 = 612,
	REG_ENTITY0613 = 613,
	REG_ENTITY0614 = 614,
	REG_ENTITY0615 = 615,
	REG_ENTITY0616 = 616,
	REG_ENTITY0617 = 617,
	REG_ENTITY0618 = 618,
	REG_ENTITY0619 = 619,
	REG_ENTITY0620 = 620,
	REG_ENTITY0621 = 621,
	REG_ENTITY0622 = 622,
	REG_ENTITY0623 = 623,
	REG_ENTITY0624 = 624,
	REG_ENTITY0625 = 625,
	REG_ENTITY0626 = 626,
	REG_ENTITY0627 = 627,
	REG_ENTITY0628 = 628,
	REG_ENTITY0629 = 629,
	REG_ENTITY0630 = 630,
	REG_ENTITY0631 = 631,
	REG_ENTITY0632 = 632,
	REG_ENTITY0633 = 633,
	REG_ENTITY0634 = 634,
	REG_ENTITY0635 = 635,
	REG_ENTITY0636 = 636,
	REG_ENTITY0637 = 637,
	REG_ENTITY0638 = 638,
	REG_ENTITY0639 = 639,
	REG_ENTITY0640 = 640,
	REG_ENTITY0641 = 641,
	REG_ENTITY0642 = 642,
	REG_ENTITY0643 = 643,
	REG_ENTITY0644 = 644,
	REG_ENTITY0645 = 645,
	REG_ENTITY0646 = 646,
	REG_ENTITY0647 = 647,
	REG_ENTITY0648 = 648,
	REG_ENTITY0649 = 649,
	REG_ENTITY0650 = 650,
	REG_ENTITY0651 = 651,
	REG_ENTITY0652 = 652,
	REG_ENTITY0653 = 653,
	REG_ENTITY0654 = 654,
	REG_ENTITY0655 = 655,
	REG_ENTITY0656 = 656,
	REG_ENTITY0657 = 657,
	REG_ENTITY0658 = 658,
	REG_ENTITY0659 = 659,
	REG_ENTITY0660 = 660,
	REG_ENTITY0661 = 661,
	REG_ENTITY0662 = 662,
	REG_ENTITY0663 = 663,
	REG_ENTITY0664 = 664,
	REG_ENTITY0665 = 665,
	REG_ENTITY0666 = 666,
	REG_ENTITY0667 = 667,
	REG_ENTITY0668 = 668,
	REG_ENTITY0669 = 669,
	REG_ENTITY0670 = 670,
	REG_ENTITY0671 = 671,
	REG_ENTITY0672 = 672,
	REG_ENTITY0673 = 673,
	REG_ENTITY0674 = 674,
	REG_ENTITY0675 = 675,
	REG_ENTITY0676 = 676,
	REG_ENTITY0677 = 677,
	REG_ENTITY0678 = 678,
	REG_ENTITY0679 = 679,
	REG_ENTITY0680 = 680,
	REG_ENTITY0681 = 681,
	REG_ENTITY0682 = 682,
	REG_ENTITY0683 = 683,
	REG_ENTITY0684 = 684,
	REG_ENTITY0685 = 685,
	REG_ENTITY0686 = 686,
	REG_ENTITY0687 = 687,
	REG_ENTITY0688 = 688,
	REG_ENTITY0689 = 689,
	REG_ENTITY0690 = 690,
	REG_ENTITY0691 = 691,
	REG_ENTITY0692 = 692,
	REG_ENTITY0693 = 693,
	REG_ENTITY0694 = 694,
	REG_ENTITY0695 = 695,
	REG_ENTITY0696 = 696,
	REG_ENTITY0697 = 697,
	REG_ENTITY0698 = 698,
	REG_ENTITY0699 = 699,
	REG_ENTITY0700 = 700,
	REG_ENTITY0701 = 701,
	REG_ENTITY0702 = 702,
	REG_ENTITY0703 = 703,
	REG_ENTITY0704 = 704,
	REG_ENTITY0705 = 705,
	REG_ENTITY0706 = 706,
	REG_ENTITY0707 = 707,
	REG_ENTITY0708 = 708,
	REG_ENTITY0709 = 709,
	REG_ENTITY0710 = 710,
	REG_ENTITY0711 = 711,
	REG_ENTITY0712 = 712,
	REG_ENTITY0713 = 713,
	REG_ENTITY0714 = 714,
	REG_ENTITY0715 = 715,
	REG_ENTITY0716 = 716,
	REG_ENTITY0717 = 717,
	REG_ENTITY0718 = 718,
	REG_ENTITY0719 = 719,
	REG_ENTITY0720 = 720,
	REG_ENTITY0721 = 721,
	REG_ENTITY0722 = 722,
	REG_ENTITY0723 = 723,
	REG_ENTITY0724 = 724,
	REG_ENTITY0725 = 725,
	REG_ENTITY0726 = 726,
	REG_ENTITY0727 = 727,
	REG_ENTITY0728 = 728,
	REG_ENTITY0729 = 729,
	REG_ENTITY0730 = 730,
	REG_ENTITY0731 = 731,
	REG_ENTITY0732 = 732,
	REG_ENTITY0733 = 733,
	REG_ENTITY0734 = 734,
	REG_ENTITY0735 = 735,
	REG_ENTITY0736 = 736,
	REG_ENTITY0737 = 737,
	REG_ENTITY0738 = 738,
	REG_ENTITY0739 = 739,
	REG_ENTITY0740 = 740,
	REG_ENTITY0741 = 741,
	REG_ENTITY_MAX,
};

enum reg_module
{
	REG_M_SYS = 0,
	REG_M_PMBUS0,
	REG_M_PMBUS1,
	REG_M_PMBUS2,
	REG_M_HDMI,
	REG_M_DP,
	REG_M_VGA,
	REG_M_AUDIO,
	REG_M_VPU0,
	REG_M_VPU1,
	REG_M_VPU2, // 10
	REG_M_VPU3,
	REG_M_APU_L,
	REG_M_APU_R,
	REG_M_DPU0,
	REG_M_DPU1,
	REG_M_DPU2,
	REG_M_DPU3,
	REG_M_DPU4,
	REG_M_DPU5,
	REG_M_DPU6, // 20
	REG_M_DPU7,
	REG_M_DMA,
	REG_M_PDMA,
	REG_M_SYS_PCIE,
	REG_M_SYS_PCIE_CFG,
	REG_M_HDMI1,
	REG_M_LVDS,
	REG_M_GPIO,
	REG_M_GPIO_OLD,
	REG_M_BL_GPIO,
	REG_M_VPU4,
	REG_M_VPU5,
	REG_M_TOP,
	REG_M_DDR0,
	REG_M_DDR1,
	REG_M_C2D_SERVICE,
	REG_M_PWM,
	REG_M_LOW_SPD_PWM_FAN,
	REG_M_LOW_SPD_TIMER,
	REG_M_BS_DDR0_0,
	REG_M_BS_DDR0_1,
	REG_M_BS_DDR1_0,
	REG_M_BS_DDR1_1,
	REG_M_BS_DDR2_0,
	REG_M_BS_DDR2_1,
	REG_M_BS_DDR3_0,
	REG_M_BS_DDR3_1,
	REG_M_DMAR,
	REG_M_DMA0,
	REG_M_DMA1,
	REG_M_DMA2,
	REG_M_DMA3,
	REG_M_VF_DMA,
	REG_M_HDMI0,
	REG_M_HDDP0,
	REG_M_HDDP1,
	REG_M_HDDP2,
	REG_M_GPU2RAW,
	REG_M_AUDIO_1,
	REG_M_AUDIO_2,
	REG_M_AUDIO_3,
	REG_M_AUDIO_4,
	REG_M_AUDIO_CTL,
	REG_M_SCP,
	REG_M_GPU_COMMON_REG0,
	REG_M_GPU_COMMON_REG1,
	REG_M_GPU_COMMON_REG2,
	REG_M_GPU_COMMON_REG3,
	REG_M_PPU_LOWSP,
	REG_M_PPU_SLC0_1,
	REG_M_PPU_INNOLINK0,
	REG_M_PPU_INNOLINK1,
	REG_M_PPU_VDEC0,
	REG_M_PPU_VDEC1,
	REG_M_PPU_VDEC2,
	REG_M_PPU_AISR,
	REG_M_PPU_AUDIO,
	REG_M_PPU_VENC0,
	REG_M_PPU_VENC1,
	REG_M_PPU_VENC2,
	REG_M_PPU_PDP0,
	REG_M_PPU_PDP1,
	REG_M_PPU_PDP2,
	REG_M_PPU_HDMI0,
	REG_M_PPU_HDMI1,
	REG_M_PPU_HDDP0,
	REG_M_PPU_HDDP1,
	REG_M_PPU_HDDP2,
	REG_M_PPU_DMA0_1,
	REG_M_PPU_VGA,
	REG_M_PPU_PCIE,
	REG_M_PPU_GPU,
	REG_M_PPU_GDDR0,
	REG_M_PPU_GDDR1,
	REG_M_PPU_GDDR4,
	REG_M_PPU_GDDR5,
	REG_M_PPU_GDDR8,
	REG_M_PPU_GDDR10,
	REG_M_PPU_GDDR12,
	REG_M_PPU_GDDR2,
	REG_M_PPU_GDDR3,
	REG_M_PPU_GDDR6,
	REG_M_PPU_GDDR7,
	REG_M_PPU_GDDR9,
	REG_M_PPU_GDDR11,
	REG_M_PPU_GDDR13,
	REG_M_INNOLINK0,
	REG_M_INNOLINK1,
	REG_M_MAX,
};

enum hal_irq_id
{
	HAL_INTERRUPT_DPU_0 = 0, /*0*/
	HAL_INTERRUPT_DPU_1,
	HAL_INTERRUPT_DPU_2,
	HAL_INTERRUPT_DPU_3,
	HAL_INTERRUPT_DPU_4,
	HAL_INTERRUPT_DPU_5,
	HAL_INTERRUPT_DPU_6,
	HAL_INTERRUPT_DPU_7,
	HAL_INTERRUPT_GPU_0, /*8*/
	HAL_INTERRUPT_GPU_1,
	HAL_INTERRUPT_GPU_2,
	HAL_INTERRUPT_GPU_3,
	HAL_INTERRUPT_CODEC_0, /*12*/
	HAL_INTERRUPT_CODEC_1,
	HAL_INTERRUPT_CODEC_2,
	HAL_INTERRUPT_CODEC_3,
	HAL_INTERRUPT_AUDIO, /*16*/
	HAL_INTERRUPT_HDMI,
	HAL_INTERRUPT_DP,
	HAL_INTERRUPT_AXI_DMA, /*19*/
	HAL_INTERRUPT_AXI_DMA_CHAN,
	HAL_INTERRUPT_PCIE_DMA, /*21*/
	HAL_INTERRUPT_APU_0,	/*22*/
	HAL_INTERRUPT_APU_1,
	HAL_INTERRUPT_HDMI0,
	HAL_INTERRUPT_HDMI1,
	HAL_INTERRUPT_LVDS,
	HAL_INTERRUPT_DEBUG,
	HAL_INTERRUPT_CODEC_4,
	HAL_INTERRUPT_CODEC_5,
	HAL_INTERRUPT_MAILBOX,
	HAL_INTERRUPT_DMAR,
	HAL_INTERRUPT_DMAR_CHAN,
	HAL_INTERRUPT_OVERHEAT,
	HAL_INTERRUPT_AUDIO_0,
	HAL_INTERRUPT_AUDIO_1,
	HAL_INTERRUPT_AUDIO_2,
	HAL_INTERRUPT_AUDIO_3,
	HAL_INTERRUPT_AUDIO_4,
	HAL_INTERRUPT_SR,
	HAL_INTERRUPT_GPU_AND_CPU0,
	HAL_INTERRUPT_GPU_AND_CPU1,
	HAL_INTERRUPT_PMU0,
	HAL_INTERRUPT_PMU1,
	HAL_INTERRUPT_PMU2,
	HAL_INTERRUPT_NOC,
	HAL_INTERRUPT_HDMI_0, /* HDMI */
	HAL_INTERRUPT_HDMI_1,
	HAL_INTERRUPT_HDDP_0, /* HDMI or DP */
	HAL_INTERRUPT_HDDP_1,
	HAL_INTERRUPT_HDDP_2,
	HAL_INTERRUPT_VGA,
	HAL_INTERRUPT_AXI_MONITOR,
	HAL_INTERRUPT_HDMI_RX,
	HAL_INTERRUPT_VIDEO_WR,
	HAL_INTERRUPT_GPU0_0,
	HAL_INTERRUPT_GPU0_1,
	HAL_INTERRUPT_GPU0_2,
	HAL_INTERRUPT_GPU0_3,
	HAL_INTERRUPT_GPU0_4,
	HAL_INTERRUPT_GPU0_5,
	HAL_INTERRUPT_GPU0_6,
	HAL_INTERRUPT_GPU0_7,
	HAL_INTERRUPT_GPU0_8,
	HAL_INTERRUPT_GPU0_9,
	HAL_INTERRUPT_GPU0_10,
	HAL_INTERRUPT_GPU0_11,
	HAL_INTERRUPT_GPU0_12,
	HAL_INTERRUPT_GPU0_13,
	HAL_INTERRUPT_GPU0_14,
	HAL_INTERRUPT_GPU0_15,
	HAL_INTERRUPT_AXI_DMA0_S1_CH,  /* chan INT */
	HAL_INTERRUPT_AXI_DMA0_S1_CMN, /* DMA common register INT status */
	HAL_INTERRUPT_AXI_DMA0_S2_CH,  /* chan INT */
	HAL_INTERRUPT_AXI_DMA0_S2_CMN, /* DMA common register INT status */
	HAL_INTERRUPT_AXI_DMA1_S1_CH,  /* chan INT */
	HAL_INTERRUPT_AXI_DMA1_S1_CMN, /* DMA common register INT status */
	HAL_INTERRUPT_AXI_DMA1_S2_CH,  /* chan INT */
	HAL_INTERRUPT_AXI_DMA1_S2_CMN, /* DMA common register INT status */
	HAL_INTERRUPT_SMMU,
	HAL_INTERRUPT_RAW_PCIE,
	HAL_INTERRUPT_RAW_GPU,
	HAL_INTERRUPT_INNO_LINK1,
	HAL_INTERRUPT_INNO_LINK,
	HAL_INTERRUPT_COUNT_MAX, /*24*/
};

enum hwinfo_item_id {
	MAGIC_NUM,
	VERSION,
	CRC16_DATA,
	HW_POWER_SUPPLY,
	HW_PCIE_LANES,
	SW_MEM_BAR_SIZE,
	DATA_LENGTH,
	HW_SN_LEGAL,
	GPU_TYPE,
	DOUBLE_CHIP,
	HW_PCIE_SPEED,
	HW_SUBSYSTEM_ID,
	HW_SUBSYSTEM_VENDOR_ID,
	SW_VF_MEM_BAR_SIZE,
	SW_VF_MEM_BAR_NUM,
	SUB_VERSION,
	CRC16_HEADER,
	HW_PRODUCT_NUMBER,
	HW_PCB_VERSION,
	HW_MEM_CHIP,
	HW_MEM_SPEED,
	HW_MEM_TYPE,
	HW_MEM_UNIT_NUMS,
	HW_MEM_UNIT_SIZE_GB,
	HW_MEM_SIDE_FLAG,
	HW_DISPLAY_HDMI_NUMS,
	HW_DISPLAY_DP_NUMS,
	HW_DISPLAY_VGA_NUMS,
	HW_DISPLAY_DVI_NUMS,
	HW_DISPLAY_LVDS_NUMS,
	HW_DISPLAY_DP2VGA_EN,
	HW_DISPLAY_HDMI2DVI_EN,
	HW_HDMI_DP_EN,
	HW_LVDS_VGA_MISC_EN,
	HW_4K_EN,
	HW_DUAL_LINK,
	HW_THERMAL_TYPE,
	HW_DISPLAY_OUTPUT_EN,
	HW_MEM_DQ_MAP_MODE,
	HW_MEM_MANUFACTURER_ID,
	HW_MEM_ATTRIBUTE,
	HW_DP2VGA_CHIP,
	HW_DP2VGA_I2C_ID,
	HW_DP2VGA_I2C_ADDR,
	HW_VGA_RESET_PIN,
	HW_VGA_HARD_RESET,
	HW_TSENSOR_CHIP,
	HW_TSENSOR_NUM,
	HW_TSENSOR_I2C_ID,
	HW_TSENSOR_I2C_ADDR,
	HW_PWRCTRL_CHIP,
	HW_PWRCTRL_NUM,
	HW_PWRCTRL_I2C_ID,
	HW_PWRCTRL_I2C_ADDR,
	HW_VOLTCTRL_CHIP,
	HW_VOLTCTRL_NUM,
	HW_VOLTCTRL_I2C_ID,
	HW_VOLTCTRL_I2C_ADDR,
	HW_VOLTCTRL_CHAN0,
	HW_VOLTCTRL_CHAN1,
	HW_VOLTCTRL_CHAN2,
	HW_POWER_CHIP,
	HW_POWER_NUM,
	HW_POWER_I2C_ID,
	HW_POWER_I2C_ADDR,
	HW_SN,
	SN,
	HW_DISPLAY_CHIP1_HDMI_NUMS,
	HW_DISPLAY_CHIP1_DP_NUMS,
	HW_DISPLAY_CHIP1_VGA_NUMS,
	HW_DISPLAY_CHIP1_DVI_NUMS,
	HW_DISPLAY_CHIP1_LVDS_NUMS,
	HW_DISPLAY_CHIP1_DP2VGA_EN,
	HW_DISPLAY_CHIP1_HDMI2DVI_EN,
	HW_CHIP1_HDMI_DP_EN,
	HW_CHIP1_4K_EN,
	HW_CHIP1_DP2VGA_I2C_ID,
	HW_CHIP1_DP2VGA_I2C_ADDR,
	HW_CHIP1_VGA_RESET_PIN,
	HW_CHIP1_VGA_HARD_RESET,
	HW_PWRCTRL_CHIP1_I2C_ID,
	HW_PWRCTRL_CHIP1_I2C_ADDR,
	HW_VOLTCTRL_CHIP1_I2C_ID,
	HW_VOLTCTRL_CHIP1_I2C_ADDR,
	HW_VOLTCTRL_I2C_ID_1,
	HW_VOLTCTRL_I2C_ADDR_1,
	HW_VOLTCTRL_CHIP1_I2C_ID_1,
	HW_VOLTCTRL_CHIP1_I2C_ADDR_1,
	HW_POWER_CHIP1_I2C_ID,
	HW_POWER_CHIP1_I2C_ADDR,
	HW_POWER_I2C_ID_1,
	HW_POWER_I2C_ADDR_1,
	HW_POWER_CHIP1_I2C_ID_1,
	HW_POWER_CHIP1_I2C_ADDR_1,
	HW_VOLTCTRL1_CHAN0,
	HW_VOLTCTRL1_CHAN1,
	HW_VOLTCTRL1_CHAN2,
	HW_FANCTRL_7BIT_ADDR,
	HW_VOLTCTRL_7BIT_ADDR,
	SW_PCIE_CLASS_CODE_SEL,
	HW_PRODUCT_TYPE,
	HW_ODM_GEN_VERSION,
	HW_FANCTRL_ENABLE,
	HW_FANCTRL_CHIP,
	HW_FANCTRL_CHIP_NUM,
	HW_FANCTRL_PWM_ID,
	HW_FANCTRL_PMBUS_ADDR,
	HW_FANCTRL_FRQ,
	HW_FANCTRL_PWM_DIRECTION,
	HW_MEM_LAYOUT,
	HW_GPIO_BITMAP_EN,
	HW_GPIO_BITMAP_LEVEL,
	HW_ODM_VENDOR,
	HW_ODM_PRODUCE_BATCH,
	HW_PWM_FUNC,
	HW_DISPLAY_OUTPUT_MODE,
	HW_BACKLIGHT_MODE,
	HW_GPIO_FUNC_MODE,
	GPIO_MEM_MANUFACTURER_ID,
	GPIO0_MEM_CHIP,
	GPIO1_MEM_CHIP,
	GPIO2_MEM_CHIP,
	GPIO3_MEM_CHIP,
	/* add  hwinfo tab new field before  HW_INFO_TAB_MAX */
	HW_INFO_TAB_MAX, //enum HWINFO_TAB_ID

	CUS_MAGIC_NUM,
	CUS_DATA_LENGTH,
	CUS_CRC16_DATA,
	CUS_VERSION,
	CUS_GPU_TYPE,
	GPU_ID,
	CUS_SUB_VERSION,
	PRODUCT_TYPE,
	CUS_CRC16_HEADER,
	PWR_CTRL_MODE,
	NORMAL_MODE_FREQ,
	POWERSAVE_MODE_FREQ,
	PERFORMANCE_MODE_FREQ,
	DYN_MODE_MODE_FREQ,
	POWERSAVE_MODE_VDD_CORE_VOL,
	POWERSAVE_MODE_DATABUS_FREQ,
	POWERSAVE_MODE_DDR_SPEED,
	POWERSAVE_MODE_PCIE_SPEED,
	DYN_TEMP_START_TRIP,
	DYN_TEMP_STEP,
	DYN_FREQ_MAX,
	DYN_FREQ_STEP,
	DYN_FREQ_MIN,
	USER_LEVEL_SET_SUPPORT,
	USER_LEVEL_FREQ_MAX,
	USER_LEVEL_FREQ_STEP,
	USER_LEVEL_FREQ_MIN,
	VOLT_SET_SUPPORT,
	FAN_MODE,
	FAN_SPEED,

	PERFORMANCE_MODE_VOLT,
	DYN_UPDATE_VOL_ENABLE,

	DYN_LPC_VERSION,
	DYN_LPC_DBUS_FREQ,
	DYN_LPC_GPU_UTILS,

	HDMI0_TYPE,
	HDMI1_TYPE,
	DP_TYPE,
	LVDS_TYPE,
	MAIN_SCREEN,
	HW_4K_ENABLE,
	HDMI0_EDID_MODE,
	HDMI1_EDID_MODE,
	DP_EDID_MODE,
	LVDS_EDID_MODE,
	ZOOM_MODE,
	DUAL_LINK,
	HDMI0_EYE_DIAGRAM,
	HDMI1_EYE_DIAGRAM,
	DP_EYE_DIAGRAM,
	VGA_TYPE,
	VGA_EDID_MODE,
	HDMI0_EDID1,
	HDMI0_EDID2,
	HDMI1_EDID1,
	HDMI1_EDID2,
	DP_EDID1,
	DP_EDID2,
	LVDS_EDID1,
	LVDS_EDID2,
	DPU_MATCH,
	VGA_EDID1,
	VGA_EDID2,
	SCR_PWM_FREQUENCY,
	SCR_DEF_BRIGHTNESS,
	SCR_BRIGHTNESS_LEVEL,
	SCR_MAX_DUTY_CYCLE,
	SCR_MIN_DUTY_CYCLE,
	OVER_TEMP_ENABLE,
	OVER_TEMP_CHIP,
	OVER_TEMP_BOARD,
	LIMIT_FREQ_ENABLE,
	LIMIT_TEMP_CHIP,
	LIMIT_TEMP_BOARD,
	LIMIT_FREQ_THD,
	DISPLAY_HPD_MODE,
	HDMI0_ANACFG,
	HDMI1_ANACFG,
	VGA_ANACFG,
	DP_ANACFG,
	CU_ODM_GEN_VERSION,
	HDMI0_HDMI1_CLK_CUR,
	NORMAL_FREQ_MIN,
	NORMAL_FREQ_MAX,
	PERFORMANCE_FREQ_MIN,
	PERFORMANCE_FREQ_MAX,
	PCIE_DA_ENABLE,
	PCIE_DA_IDO_V720SEL,
	PCIE_DA_EXSWING_EN,
	PCIE_DA_EXSWING_ISEL,
	PCIE_ASPM_CAP_CTRL,
	MCU_DISPLAY_EN,
	WM_FLAG,
	OTA_DISABLE,
	WM_DENSITY,
	WM_DENSITY_1,
	WM_GRADIENT,
	WM_RGBA,
	WM_TIME_ENABLE,
	WM_TIME_REFRESH,
	WM_ALPHA,
	WM_STRING,
	PCIE_SEL_LEG_SWING,
	PCIE_TX_PRESET_VALUE,
	PCIE_BAR_ATTR,
	DDR_RETRAIN_TEMP_DIFF,

	/* add custom tab new field before CUSTOM_TAB_MAX  */
	CUSTOM_TAB_MAX,
}; //end of enum CUSTOM_TAB_ID

enum hwinfo_item_type {
	HWINFO_TYPE_STR,           /* 字符串 */
	HWINFO_TYPE_BYTE8,         /* 占用一个字节 */
	HWINFO_TYPE_BYTE16,        /* 占用两个字节 */
	HWINFO_TYPE_BYTE32,        /* 占用四个字节 */
	HWINFO_TYPE_MULT_BYTE,     /* 占用多个字节但内容不是字符串 */
	HWINFO_VAL_TYPE_MAX
};

struct hwinfo_item {
	char *name;
	enum hwinfo_item_id hwinfo_id;
	enum hwinfo_item_type hwinfo_type;
	unsigned short offset;
	unsigned char val_size;
	unsigned int val32;
	int def_val32;
	char *strval;
};

struct gpu_info_hwinfo{
	uint32_t fan_speed;
	uint32_t vram_type;
	uint32_t mem_width;
	uint32_t mem_nums;
	uint32_t vram_clock;
	uint32_t gpu_voltage;
	uint32_t gpu_power;
	uint32_t link_speed;
	uint32_t link_width;
};

//mcufw comm start
#define MCUFW_COMM_MSG_CNT                        20    //msg size default 20 byte
//电压交互命令
#define MCUFW_COMM_CMD_VOL_SET                    0x1
#define MCUFW_COMM_CMD_VOL_GET                    0x2
#define MCUFW_COMM_CMD_VOL_GET_EXT                0x3
#define MCUFW_COMM_CMD_VOL_NOTIFY				  0x4
#define MCUFW_COMM_CMD_VOL_GET_VER				  0x5
#define MCUFW_COMM_CMD_VOL_GET_CAPABILITIES		  0x6

#define MCUFW_VOL_CAPABILITIES_IS_DIGITAL_FLAG    (0x1 << 0)

//风扇交互命令
#define MCUFW_COMM_CMD_SET_FAN                    0x1
#define MCUFW_COMM_CMD_GET_FAN                    0x2
#define MCUFW_COMM_CMD_GET_FAN_VER				  0x3
#define MCUFW_COMM_CMD_SET_PCIE_SPEED			  0x4

//温度交互命令
#define MCUFW_COMM_CMD_OVER_TEMP                  0x1
#define MCUFW_COMM_CMD_DDR_RETRAIN_ENABLE         0x2
#define MCUFW_COMM_CMD_DDR_RETRAIN_TEMP_THRESHOL  0x3
#define MCUFW_COMM_CMD_TEMP_GET_VER               0x4

//mcufw信息上报交互命令
#define MCUFW_COMM_CMD_MCUFW_REPORT               0x1
#define MCUFW_COMM_CMD_MCUFW_RELEASE_TIME         0x2
#define MCUFW_COMM_CMD_MCUFW_BUILD_VER            0x3
#define MCUFW_COMM_CMD_MCUFW_GET_SUBVER           0x4

//vbios快速启动交互命令
#define MCUFW_COMM_CMD_SET_RES_POWER              0x1
#define MCUFW_COMM_CMD_WRITE_VRAM                 0x2

#define MCUFW_COMM_MODULE_LEN (32)
enum HAL_MCUFW_COMM_MODULE {
	MCUFW_COMM_MODULE_VOLTAGE = 0,
	MCUFW_COMM_MODULE_FAN,
	MCUFW_COMM_MODULE_TEMP,
	MCUFW_COMM_MODULE_MCUFW,
	MCUFW_COMM_MODULE_VBIOS,
	MCUFW_COMM_MODULE_BMC,
	MCUFW_COMM_MODULE_EXPRESS,
	MCUFW_COMM_MODULE_DEBUG,
	MCUFW_COMM_MODULE_MAX,
};
//mcufw comm end

typedef struct plat_data {
	unsigned int dev_idx;
	int dev_nums;
	int pdev_idx;
	//TODO: delete this pointer later
	struct dev_rsrc *pdev_rsrc;
}plat_data_t;

struct inno_hal_dma_para {
	char *dma_name;
	char *dma_type;
	inno_dev *dma_dev;
};

struct hal_gpu_feature {
	/* gpu feature for umd */
	uint32_t device_id;
	uint32_t spu_num;
	uint32_t clusters_num;
	uint32_t spu1_clusters;
	uint32_t tpu_buffer_lookup_size_kb;
	uint32_t sgl_max_texture_buffer_size;
	uint32_t gles_max_texture_buffer_size;
};

struct pmbus_funcs{
	void (*set_pmbus_freq)(inno_i2c_adapter *adap, uint32_t baudrate);
};

#define BASE_ADDR_PCIE_ACCESS_HOST            (0x1800000000ULL)
#define BASE_ADDR_HOST_RAM                    (0x0ULL)

// hal.c
chip_type_e fh2m_hal_get_chiptype(inno_dev *dev);

int fh2m_hal_get_dev_nums(inno_dev *dev, dev_module_e devmod);

void *fh2m_hal_get_reg_addr(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity);

int fh2m_hal_reg_read8(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint8_t *return_val);

int fh2m_hal_reg_read16(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint16_t *return_val);

int fh2m_hal_reg_read32(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint32_t *return_val);

int fh2m_hal_reg_write8(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint8_t val);

int fh2m_hal_reg_write16(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint16_t val);

int fh2m_hal_reg_write32(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint32_t val);

int fh2m_hal_reg_write32_offset(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint32_t wr_offset, uint32_t val);

int fh2m_hal_reg_read32_offset(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint32_t wr_offset, uint32_t * return_val);

int fh2m_hal_reg_entity_addr(inno_dev *dev, enum reg_module reg_m, enum reg_entity entity, uint32_t *offset);

uint32_t fh2m_hal_reg_map_size_get(inno_dev *dev, enum reg_module reg_m);

uint32_t fh2m_hal_get_pll(inno_dev* dev, pll_module_e module);

void fh2m_hal_set_pll(inno_dev* dev, pll_module_e module, uint32_t pll);

void fh2m_hal_audio_set_pll(inno_dev *dev, enum reg_module reg_m, uint32_t reg1, uint32_t reg2);

void hal_irq_init(void);

int fh2m_hal_set_irq_handler(inno_dev* dev, int hal_irq_id, void (*handler_func)(void*), void*data);

int fh2m_hal_dev_enable_irq(inno_dev* dev, int hal_irq_id);
int fh2m_hal_clean_irq_status(inno_dev* dev, int hal_irq_id);
int fh2m_hal_dev_disable_irq(inno_dev* dev, int hal_irq_id);

uint32_t fh2m_hal_get_gpu_irq_by_mode(inno_dev* dev, uint32_t index);

char * fh2m_hal_get_chip_name(inno_dev* dev);

char * fh2m_hal_get_chip_alias(inno_dev* dev);

bool fh2m_hal_is_low_power_chip(inno_dev* dev);

uint32_t fh2m_hal_init_vram_total_size(inno_dev *dev);

int fh2m_hal_get_multi_memory_regions_en(void);

// hal_vram.c
bool fh2m_hal_has_inv_mem(inno_dev *dev);

uint64_t fh2m_hal_get_inv_mem_total_size(inno_dev *dev);

/**
 * fh2m_hal_vram_alloc: alloc continous vram api
 * @dev        : pcie device
 * @role_tar   : vram role
 * @cpu_access : alloc visible or invisible memory
 * @alloc_size : memory alloc size
 * @flags      : flags control mem alloc, see hal_physmem_alloc_flags
 * return      : 0 faled , > 0 vram dev address
 * note        : this func cannot alloc non-contig vram, please do not use HAL_PHYSMEM_ALLOC_NON_CONTIG flag
 */
uint64_t fh2m_hal_vram_alloc(inno_dev* dev, struct role_target *role_tar, bool cpu_access, uint64_t alloc_size, uint64_t flags);

int fh2m_hal_vram_free(inno_dev* dev, struct role_target *role_tar, uint64_t base);

uint64_t fh2m_hal_get_vpu_vram_zone_base(inno_dev *dev, uint8_t id);

int fh2m_hal_inv_vram_free(inno_dev *dev, struct role_target *role_tar, uint64_t base);

int fh2m_hal_init_reserved_vram(inno_dev *dev);

int fh2m_hal_deinit_reserved_vram(inno_dev *dev);

uint64_t fh2m_hal_get_ddrbase(inno_dev* dev);

uint64_t fh2m_hal_get_ddr_bar_len(inno_dev* dev);

uint64_t fh2m_hal_get_vram_dev_base(inno_dev* dev);

uint64_t fh2m_cpu_paddr_to_dev_paddr(inno_dev* dev, uint64_t cpu_paddr);

uint64_t fh2m_dev_paddr_to_cpu_paddr(inno_dev* dev, uint64_t dev_paddr);

uint64_t fh2m_cpu_paddr_to_pcie_paddr(inno_dev* dev, uint64_t cpu_paddr);

uint64_t fh2m_cpu_paddr_to_gtt_paddr(inno_dev* dev, uint64_t cpu_paddr);

uint64_t fh2m_gtt_paddr_to_cpu_paddr(inno_dev* dev, uint64_t gtt_paddr);

void fh2m_hal_get_vram_layout(inno_dev* dev, void *zones, int *zone_nums);

uint64_t fh2m_hal_get_maxsize_by_role(inno_dev* dev, enum vram_alloc_role vram_role);

void __iomem* fh2m_hal_get_sys_reg_base(inno_dev* dev);

void __iomem* fh2m_hal_get_bar0_reg_base(inno_dev* dev);

resource_size_t fh2m_hal_get_bar0_reg_paddr(inno_dev* dev);

bool fh2m_hal_is_sharing_gpu_heap(inno_dev *dev);

void fh2m_hal_pdp_sys_reset(inno_dev *dev, enum reg_module reg_m);

int fh2m_hal_pdp_video_set(inno_dev *dev, int pdp_num, enum reg_module reg_m);
int fh2m_hal_pdp_video_set_noclock(inno_dev *dev, int pdp_num, enum reg_module reg_m);
int fh2m_hal_subpdp_video_set(inno_dev *dev, int pdp_num, enum reg_module reg_m);

uint32_t fh2m_hal_get_gpu_core_nums(void);
uint32_t fh2m_hal_get_gpu_mc_mode(void);
uint32_t fh2m_hal_get_gpu_core_index(void);

bool fh2m_hal_get_nulldisplay(void);

int fh2m_hal_get_nulldisplay_drm_pipe_num(void);

bool fh2m_hal_get_dpu_hw_support_gtt(void);

#if defined(__G0M_SOC__)
int fh2m_hal_get_prohibit_umd_gtt_alloc(void);
#endif

void fh2m_hal_set_gPVRDebugLevel(uint32_t ui32DebugLevel);

uint32_t fh2m_hal_get_gPVRDebugLevel(void);

uint32_t fh2m_hal_get_inno_gfp_kernel(void);

unsigned int fh2m_hal_get_s_dpu_debug(void);

unsigned int fh2m_hal_get_s_max_width(void);
unsigned int fh2m_hal_get_s_max_height(void);

unsigned int fh2m_hal_get_s_vkms_width(void);
unsigned int fh2m_hal_get_s_vkms_height(void);

int fh2m_hal_get_gpu_stat(inno_dev *dev, int gpu_id, struct vram_stats *stats);

int fh2m_hal_get_memsize(inno_dev *dev);

int fh2m_hal_get_pcie_dma_stat(inno_dev *dev);

int fh2m_hal_getflag_dp2vga(inno_dev *dev, unsigned char index);

int fh2m_hal_getflag_hdmi2dvi(inno_dev *dev, unsigned char index);

int fh2m_hal_get_chip_static_info(inno_dev *dev,char *buf,ssize_t *count);
int fh2m_hal_get_chip_gpuinfo(inno_dev *dev,char *buf,ssize_t *count);

int fh2m_hal_get_chip_dyn_info(inno_dev *dev,char *buf,ssize_t *count,bool develop_mode);

int fh2m_hal_get_driver_info(inno_dev *dev,char *buf,ssize_t *count,bool develop_mode);

int fh2m_hal_get_fw_env(inno_dev *dev,char *buf,ssize_t *count,bool develop_mode);

int fh2m_hal_trigger_mcu_intr(inno_dev *dev,int intr_type);

int fh2m_hal_get_axi_latency_info(inno_dev *dev,char *buf,ssize_t *count, int monitor_time, bool develop_mode);

int fh2m_hal_get_axi_bandwidth_info(inno_dev *dev,char *buf,ssize_t *count,int monitor_time,bool develop_mode);

int fh2m_hal_get_ddr_bandwidth_info(inno_dev *dev,char *buf,ssize_t *count,int monitor_time,bool develop_mode);

void fh2m_hal_sync_raw_intr(inno_dev *dev, int master, int value);

int fh2m_hal_set_gpu_raw_base(inno_dev *dev, uint64_t value);

void __iomem* fh2m_hal_get_mcu_reserved_vram(inno_dev *dev);

void __iomem* fh2m_hal_get_env_reserved_vram(inno_dev *dev);

int fh2m_hal_is_support_4k(inno_dev *dev);

void fh2m_hal_pmbus_nr_set(inno_dev *dev, int pmbus_id, int idr);

int fh2m_hal_pmbus_nr_get(inno_dev *dev, int pmbus_id);

uint64_t fh2m_hal_get_visable_mem_total_size(inno_dev *dev);

uint64_t fh2m_hal_get_vram_pcie_base(inno_dev *dev);
bool fh2m_is_support_update_voltage(inno_dev *dev);
int fh2m_hal_get_mod_pcie_drop_timeout(void);
void fh2m_mod_update_voltage_disable(void);
int fh2m_hal_set_voltage(inno_dev *dev, unsigned int voltage);
int fh2m_hal_get_voltage(inno_dev *dev);
void *fh2m_hal_get_gpufreq_info(inno_dev *dev);
int fh2m_hal_get_voltage_from_bmc(inno_dev *dev, uint32_t *mv);
int fh2m_hal_get_init_gpuvol(inno_dev *dev);
ssize_t fh2m_hal_get_mcufw_release_time(void* chip_ctx, char *firmware_release_time, ssize_t size);
int fh2m_hal_get_mem_chip(inno_dev *dev, char *name, ssize_t size);
int fh2m_hal_mcufw_comm_get_version(inno_dev *dev);
int fh2m_hal_init_mcufw_comm(void* chip_ctx);
int fh2m_hal_deinit_mcufw_comm(void* chip_ctx);
int fh2m_hal_mcufw_comm_msg_xfer(void *chip_ctx, enum HAL_MCUFW_COMM_MODULE module, uint32_t cmd, uint32_t *data, int size);
int fh2m_get_pwr_debug_lvl(void);
unsigned int fh2m_hal_is_enable_dyn_freq(inno_dev *dev);

int fh2m_hal_get_chip_temperature(inno_dev *dev);
int fh2m_hal_get_board_temperature(inno_dev *dev);
inno_i2c_adapter *fh2m_hal_get_pmbus_adapter(inno_dev *dev, int pmbus_id);

int fh2m_hal_set_pmbus_adapter_and_funcs(inno_dev *dev, inno_i2c_adapter *adap,
		struct pmbus_funcs *funcs, int pmbus_id);

int fh2m_hal_set_pmbus_freq(inno_dev *dev, int pmbus_id, uint32_t baudrate);

int fh2m_hal_power_ops_init(inno_dev *dev, void *set, void *get, void* switch_to);

struct volctrl_chip_hwinfo_t *fh2m_hal_get_volctrl_hwinfo(inno_dev *dev);

enum HAL_THERMAL_TYPE fh2m_hal_hw_thermal_type(inno_dev *dev);

int fh2m_hal_gettype_hdmi(inno_dev *dev, unsigned char index);
int fh2m_hal_gettype_dp(inno_dev *dev, unsigned char index);
int fh2m_hal_gettype_lvds_type(inno_dev *dev, unsigned char index);
int fh2m_hal_gettype_vga(inno_dev *dev, unsigned char index);
int fh2m_hal_select_4k(inno_dev *dev);
int fh2m_hal_hdmi_edid_mode(inno_dev *dev, unsigned char index);
int fh2m_hal_dp_edid_mode(inno_dev *dev, unsigned char index);
int fh2m_hal_lvds_edid_mode(inno_dev *dev, unsigned char index);
int fh2m_hal_vga_edid_mode(inno_dev *dev, unsigned char index);
int fh2m_hal_hdmi_edid_data(inno_dev *dev, unsigned char index, char *buf);
int fh2m_hal_dp_edid_data(inno_dev *dev, unsigned char index, char *buf);
int fh2m_hal_lvds_edid_data(inno_dev *dev, unsigned char index, char *buf);
int fh2m_hal_vga_edid_data(inno_dev *dev, unsigned char index, char *buf);
int fh2m_hal_get_vga_hard_reset(inno_dev *dev);
int fh2m_hal_get_vga_reset_pin(inno_dev *dev);
int fh2m_hal_get_dp2vga_i2c_addr(inno_dev *dev);
int fh2m_hal_get_dp2vga_i2c_id(inno_dev *dev);
int fh2m_hal_get_dp2vga_chip_type(inno_dev *dev);
int fh2m_hal_get_hdmi_dp_en_status(inno_dev *dev);
int fh2m_hal_get_lvds_nums(inno_dev *dev);
int fh2m_hal_get_dvi_nums(inno_dev *dev);
int fh2m_hal_get_vga_nums(inno_dev *dev);
int fh2m_hal_get_dp_nums(inno_dev *dev);
int fh2m_hal_get_hdmi_nums(inno_dev *dev);
int fh2m_hal_get_lvds_vga_misc_en(inno_dev *dev);
int fh2m_hal_get_dual_link(inno_dev *dev);
int fh2m_hal_hwinfo_version(inno_dev *dev);
int fh2m_hal_hwinfo_get_item(inno_dev *dev, enum hwinfo_item_id hwinfo_id, struct hwinfo_item *hwinfo_item);
bool fh2m_hal_get_hwinfo_finished_status(inno_dev *dev);
bool fh2m_hal_get_custominfo_finished_status(inno_dev *dev);
struct hal_gpu_feature fh2m_hal_get_gpu_feature(inno_dev *dev);
int fh2m_hal_get_odm_vendor(inno_dev *dev, char *buf);
int fh2m_hal_get_pcb_version(inno_dev *dev, char *buf);
int fh2m_hal_bmc_read32(inno_dev *dev, enum reg_entity entity, unsigned int *return_val);
int fh2m_hal_bmc_write32(inno_dev *dev, enum reg_entity entity, unsigned int val);

bool fh2m_hal_has_gtt_mem(inno_dev *dev);
bool fh2m_hal_has_limited_gtt_mem(inno_dev *dev);
ssize_t hal_show_pll(inno_dev *, char *buf, ssize_t count);
int fh2m_hal_set_pll_by_name(inno_dev *, char *name, uint32_t num);
int hal_pci_irq_setup(struct dev_rsrc *pdev_rsrc);
void hal_set_default_pcie_irq_mode(struct dev_rsrc *pdev_rsrc);
int hal_pci_irq_resume(struct dev_rsrc *pdev_rsrc);
void hal_pci_irq_free(struct dev_rsrc *pdev_rsrc);
int hal_get_support_regulator(inno_dev *dev);
ssize_t fh2m_hal_show_hw_info(inno_dev *dev, char *buf, ssize_t buf_size, int hwinfo_index);

void hal_hwinfo_init(struct dev_rsrc *pdev_rsrc);
void hal_hwinfo_deinit(struct dev_rsrc *pdev_rsrc);
int hal_init_all_bars(struct dev_rsrc *pdev_rsrc);
int hal_hw_init(struct dev_rsrc *pdev_rsrc);
void hal_quick_test(struct dev_rsrc *pdev_rsrc);
void hal_deep_test(struct dev_rsrc *pdev_rsrc);
void hal_mcufw_status_check(struct dev_rsrc *pdev_rsrc);
int hal_devices_register(struct dev_rsrc *pdev_rsrc);
void hal_devices_unregister(struct dev_rsrc *pdev_rsrc);
unsigned int fh2m_hal_get_mcufw_version(struct dev_rsrc *pdev_rsrc, char *buf, ssize_t size);
void fh2m_hal_get_gpu_info_hwinfo(struct dev_rsrc *pdev_rsrc, struct gpu_info_hwinfo *gpu_info);

int fh2m_hal_zoom_is_enable(inno_dev *dev, unsigned char dis_indnx);
int fh2m_hal_get_output_en(inno_dev *dev, unsigned short *val);
int fh2m_hal_get_output_mode(inno_dev *dev, unsigned char index, unsigned char *val);
int fh2m_hal_get_backlight_mode(inno_dev *dev, unsigned index, unsigned char *val);

unsigned int fh2m_hal_detect_registered_interface(inno_dev *dev);
void fh2m_hal_set_interface_nums(inno_dev *dev, int nums);
void fh2m_hal_set_interface_info(inno_dev *dev, void *ptr);
void fh2m_hal_append_interface_info(inno_dev *dev, int index, struct disp_interface_info *interface);
struct disp_interface_info *fh2m_hal_get_display_interface_info(inno_dev *dev, int index);

int fh2m_hal_get_vfid(inno_dev *dev);
uint64_t fh2m_hal_get_gtt_dev_base(inno_dev *dev);
int fh2m_hal_get_dpu_match(inno_dev *dev);
bool fh2m_hal_is_gtt_need_falling_back(inno_dev *dev);

uint64_t fh2m_hal_get_gtt_support(inno_dev *dev);
uint64_t fh2m_hal_get_smmu_support(inno_dev *dev);


#if defined(CONFIG_NUMA) && defined(__INNO_CONTAINER__)
int fh2m_bind_cpu_is_enable(inno_dev *dev);
int bind_numa_config_init(struct dev_rsrc* pdev_rsrc);
void bind_numa_config_deinit(void);
int fh2m_hal_bind_numa_cpu_config_store(inno_dev *dev, const char *buf);
int fh2m_hal_bind_numa_cpu_config_show(inno_dev *dev, char *buf);
#endif //END CONFIG_NUMA __INNO_CONTAINER__

/**************** HAL_VRAM ****************/
uint64_t fh2m_pcie_paddr_to_cpu_paddr(inno_dev* dev, uint64_t cpu_paddr);

bool fh2m_hal_is_gtt_mem(inno_dev* dev, uint64_t gtt_addr);
bool fh2m_hal_is_left_vram(inno_dev *dev, uint64_t dev_pa);
bool fh2m_hal_is_visible_vram(inno_dev *dev, uint64_t dev_pa);
bool fh2m_hal_is_invisible_vram(inno_dev *dev, uint64_t dev_pa);

int fh2m_hal_gtt_sort_enable(void);

/**************** HAL_DMA ****************/

#define HAL_PCIE_DMA_MAX_WR_CHAN 8

enum hal_dma_errno {
	HAL_DMA_OK,
#define HAL_DMA_ERROR(x) x,
#include "hal_dma_errors.h"
#undef HAL_DMA_ERROR
};

bool fh2m_hal_is_support_dmar(inno_dev *dev);

enum dma_direction {
	DMA_MEM2DEV = 0,
	DMA_DEV2MEM,
	DMA_DEV2DEV,
	DMA_MEM2MEM,
	DMA_DIRNONE,
};

typedef void (*pfn_dma_func)(void*);

void* fh2m_hal_dma_map(inno_dev *dev, bool mem2dev, void *address, size_t size);
void fh2m_hal_dma_unmap(inno_dev *dev, void *buffer);
int fh2m_hal_dma_transfer(inno_dev *dev, void **srcs, void **dsts, int *lens, int cnt, int dir, pfn_dma_func fast_cb, pfn_dma_func cb, void *cb_param);

void fh2m_hal_dma_deinit(struct dev_rsrc *pdev_rsrc);
int fh2m_hal_dma_init(struct dev_rsrc *pdev_rsrc);
int fh2m_hal_dma_resume(struct dev_rsrc *pdev_rsrc);
int fh2m_hal_dma_suspend(struct dev_rsrc *pdev_rsrc);

int hal_init_sys_bar(struct dev_rsrc *pdev_rsrc);
void hal_deinit_sys_bar(struct dev_rsrc *pdev_rsrc);

/**************** HAL GTT Alloc/Free ****************/
struct gtt_item {
	inno_page **page_array;
	uint32_t  page_num;
	/* dev address per page */
	uint64_t  *dev_addr_array;
	uint64_t  size;
	/* pointer to GTT Ext data */
	void      *priv;
};

struct gtt_item* fh2m_hal_gtt_alloc(inno_dev *dev, struct role_target *role_tar, uint64_t alloc_size, uint64_t flags);
void fh2m_hal_gtt_free(inno_dev *dev, struct role_target *role_tar, struct gtt_item *gtt_item);

/**************** HAL PHYS MEMORY Alloc/Free ****************/
enum phys_mem_type
{
	HAL_PHYS_MEM_LMA = 0,
	HAL_PHYS_MEM_LMA_INV,
	HAL_PHYS_MEM_GTT,
	HAL_PHYS_MEM_TYPE_MAX,
};

struct phys_mem_gtt_resource
{
	inno_page **page_array;
	uint32_t  page_num;
};

struct phys_mem_resource
{
	/* Device Address Array */
	uint64_t *base_array;
	uint32_t base_array_size;
	/* Memory size per Array item */
	uint64_t chunk_size;

	/* Actual size applyed for by the role*/
	uint64_t actual_size;

	enum phys_mem_type mem_type;

	/* Alloc flags */
	uint64_t flags;

	/* ptr to unique data structure for some mem type */
	void *mem_resource;

	/* ptr to data structure in innosrvkm */
	void *priv;
};

enum hal_physmem_alloc_flags
{
	/* Alloc Policy */
	HAL_PHYSMEM_ALLOC_NON_CONTIG = 0,
	HAL_PHYSMEM_ALLOC_NO_WARN,

	/* Memory Opt */
	HAL_PHYSMEM_ALLOC_ZERO_ON_ALLOC = 10,
	HAL_PHYSMEM_ALLOC_POISON_ON_ALLOC,
	HAL_PHYSMEM_ALLOC_POISON_ON_FREE,

	/* Memory Type */
	HAL_PHYSMEM_ALLOC_GTT = 20,
	HAL_PHYSMEM_ALLOC_LMA,
	HAL_PHYSMEM_ALLOC_LMA_INV,

	HAL_PHYSMEM_ALLOC_MAX,
};

#define HAL_PHYSMEM_ALLOC_FLAG_SET(flags, val) ((flags) |= (1 << (HAL_PHYSMEM_ALLOC_ ## val)))

/* only suitable for contig vram */
enum physmem_mapping_flags
{
	HAL_PHYSMEM_MAPPING_CPU_UNCACHED = 0,
	HAL_PHYSMEM_MAPPING_CPU_UNCACHED_WC,
	HAL_PHYSMEM_MAPPING_CPU_CACHED,
	HAL_PHYSMEM_MAPPING_CPU_CACHE_COCHERENT,
	HAL_PHYSMEM_MAPPING_CPU_CACHE_INCHOHERENT,
};

struct physmem_kernel_mapping
{
	void   *priv;
	void   *ptr;
	size_t size;

	enum phys_mem_type mem_type;
};

struct reg_info {
	uint32_t offset;
	uint32_t base;
};
int hal_get_reg_info(struct dev_rsrc *pdev_rsrc, enum reg_module reg_m,
	enum reg_entity entity, struct reg_info *info);

struct phys_mem_resource*
fh2m_hal_physmem_alloc(inno_dev *dev, struct role_target *role, uint64_t size, bool cpu_access, uint64_t flags);

void
fh2m_hal_physmem_free(inno_dev *dev, struct role_target *role, bool cpu_access, struct phys_mem_resource *pmr);

struct physmem_kernel_mapping*
fh2m_hal_physmem_kernel_mapping(inno_dev *dev, struct phys_mem_resource *pmr, enum physmem_mapping_flags mapping_flags);

void
fh2m_hal_physmem_kernel_unmapping(inno_dev *dev, struct physmem_kernel_mapping *mapping);

/**************** HAL DMA Mem Pool ****************/
int fh2m_hal_vram_dma_mem_pool_init(inno_dev *dev);

void fh2m_hal_vram_dma_mem_pool_deinit(inno_dev *dev);

uint64_t fh2m_hal_vram_dma_mem_pool_alloc(inno_dev *dev, uint64_t size, void **priv);

void fh2m_hal_vram_dma_mem_pool_free(inno_dev *dev, uint64_t address, void *priv);

void* fh2m_hal_vram_dma_mem_pool_dev_paddr_to_virt_addr(inno_dev *dev, uint64_t dev_addr);

int fh2m_hal_bmc_read32(inno_dev *dev, enum reg_entity entity, unsigned int *return_val);

int fh2m_hal_bmc_write32(inno_dev *dev, enum reg_entity entity, unsigned int val);

int hal_bmc_init(void *chip_ctx);

void hal_bmc_deinit(void *chip_ctx);

/* for PPU module interfaces
 * (power process unit) */
#define PPU_CONTROL_MODULE_LIST \
    P(LOWSPEED)	\
    P(SLC0_1)	\
    P(INNOLINK0)	\
    P(INNOLINK1)	\
    P(VDEC0)	\
    P(VDEC1)	\
    P(VDEC2)	\
    P(AISR)	\
    P(AUDIO)	\
    P(VENC0)	\
    P(VENC1)	\
    P(VENC2)	\
    P(PDP0)	\
    P(PDP1)	\
    P(PDP2)	\
    P(HDMI0)	\
    P(HDMI1)	\
    P(HDDP0)	\
    P(HDDP1)	\
    P(HDDP2)	\
    P(DMA0_1)	\
    P(VGA)	\
    P(PCIE)	\
    P(GPU)

typedef enum{
#define P(name) CTR_ ## name ## _PPU_ID,
	PPU_CONTROL_MODULE_LIST
    PPU_ID_MAX
}	ppu_control_module_e;


/* ============================ g3 add =========================== */
void hal_chip_hw_deinit(struct dev_rsrc *pdev_rsrc);
bool fh2m_hal_get_gpu_raw_enabled(inno_dev *dev);
int fh2m_hal_raw_sync_init(inno_dev *dev, uint64_t value);
void hal_set_register_write_read_sync(inno_dev *dev, int mode);
bool fh2m_hal_set_module_power_on(inno_dev *dev, uint32_t id);
bool fh2m_hal_set_module_power_off(inno_dev *dev, uint32_t id);
bool fh2m_hal_set_module_power_reset(inno_dev *dev, uint32_t id, bool auto_on);
bool fh2m_hal_set_module_clk_off(inno_dev *dev, uint32_t id);
bool fh2m_hal_set_module_flr_reset(inno_dev *dev, uint32_t id);
int fh2m_hal_get_innolink_chip_id(inno_dev *dev);
int fh2m_hal_set_gtt_write_mask_statistics(inno_dev *dev, int gtt_aw_count);
int fh2m_hal_get_gtt_write_mask_statistics(inno_dev *dev, char *buf, ssize_t *count);
int fh2m_hal_set_smmu_ops(inno_dev *dev, void *ops);
void* fh2m_hal_get_smmu_ops(inno_dev * dev);
int hal_efuse_init(void *chip_ctx);

void hal_efuse_deinit(void *chip_ctx);

void fh2m_hal_set_dev_node(inno_dev *dev, void *dev_node);

/**
 * @brief Restore the original GTT map, gtt map to host mem
 *
 * return:
 * if success,it will return 0;if fail,it will return minus number.
  */
int hal_gtt_unmap_to_peer(inno_dev *map_dev, inno_dev *cfg_dev);

/**************** INNO ML ****************/
int fh2m_hal_get_and_deal_bmc_info(inno_dev *dev, void *private_data, uint32_t *val);
void fh2m_hal_get_driver_version(char *driverversion, ssize_t size);

#endif
