/*
 * @File        pvr_sync_api.h
 * @Title       Kernel driver for Android's sync mechanism
 * @Codingstyle LinuxKernel
 * @Copyright   Copyright (c) Imagination Technologies Ltd. All Rights Reserved
 * @License     Dual MIT/GPLv2
 *
 * The contents of this file are subject to the MIT license as set out below.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 ("GPL") in which case the provisions
 * of GPL are applicable instead of those above.
 *
 * If you wish to allow use of your version of this file only under the terms of
 * GPL, and not to allow others to use your version of this file under the terms
 * of the MIT license, indicate your decision by deleting the provisions above
 * and replace them with the notice and other provisions required by GPL as set
 * out in the file called "GPL-COPYING" included in this distribution. If you do
 * not delete the provisions above, a recipient may use your version of this file
 * under the terms of either the MIT license or GPL.
 *
 * This License is also included in this distribution in the file called
 * "MIT-COPYING".
 *
 * EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
 * PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
 * COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef _PVR_SYNC_API_H
#define _PVR_SYNC_API_H

#include "img_types.h"

int pvr_sync_api_init(void *file_handle, void **api_priv);
int pvr_sync_api_deinit(void *api_priv, bool is_sw);
int pvr_sync_api_rename(void *api_priv, void *user_data);
int pvr_sync_api_force_sw_only(void *api_priv, void **api_priv_new);
int pvr_sync_api_sw_create_fence(void *api_priv, void *user_data);
int pvr_sync_api_sw_create_fence_without_timeline(void *api_priv, void *user_data);
int pvr_sync_api_sw_signal_fence(void *api_priv, void *user_data);
int pvr_sync_api_sw_inc(void *api_priv, void *user_data);
int pvr_sync_api_force_exp_only(void *api_priv, void *api_data);
int pvr_sync_api_create_export_fence(void *api_priv, void *user_data);

int pvr_sync_update_timestamp(int fd1, int fd2, int merge);

struct file;

int pvr_sync_ioctl_init(void);
void pvr_sync_ioctl_deinit(void);
void *pvr_sync_get_api_priv(struct file *file);
struct file *pvr_sync_get_file_struct(void *file_handle);

#endif /* _PVR_SYNC_API_H */
